@@ -1,22 +1,35 @@
 //! Parses and manipulates problem yaml
 
-use crate::mapm_result::MapmErr;
-use crate::mapm_result::MapmErr::*;
-use crate::mapm_result::MapmResult;
-use crate::mapm_result::MapmResult::*;
+use crate::result::MapmErr;
+use crate::result::MapmErr::*;
+use crate::result::MapmResult;
+use crate::result::MapmResult::*;
 
 use crate::template::Template;
 use Filter::*;
 use FilterAction::*;
 use Views::*;
 
-use linked_hash_map::LinkedHashMap;
+use std::fs;
+use std::path::Path;
 
-use strict_yaml_rust::StrictYamlLoader;
+use std::collections::HashMap;
 
-pub type Vars = LinkedHashMap<String, String>;
-pub type Solutions = Vec<LinkedHashMap<String, String>>;
+use serde::{Deserialize, Serialize};
 
+use serde_yaml::Value;
+
+pub type Vars = HashMap<String, String>;
+pub type Solutions = Vec<HashMap<String, String>>;
+
+#[derive(Debug, Serialize, Deserialize)]
+struct SerializedProblem {
+    pub solutions: Solutions,
+    #[serde(flatten)]
+    pub vars: HashMap<String, Value>,
+}
+
+#[derive(Debug, PartialEq, Clone)]
 pub struct Problem {
     pub vars: Vars,
     pub solutions: Solutions,
@@ -28,6 +41,7 @@ pub struct Problem {
 ///
 /// For the Gt, Lt, Ge, Le conditions, whoever is working with the problem files (either through CLI utils or a GUI) is responsible for ensuring that any key being compared with Gt, Lt, Ge, Le is a non-negative integer
 
+#[derive(Clone)]
 pub enum Filter {
     Exists { key: String },
     Eq { key: String, val: String },
@@ -40,18 +54,47 @@ pub enum Filter {
 /// Defines the actions for filtering problems
 ///
 /// Positive means "keep if this condition is satisfied", negative means "keep if this condition is **not** satisfied"
-
+#[derive(Clone)]
 pub enum FilterAction {
     Positive(Filter),
     Negative(Filter),
 }
 
 /// Views *only* shown strings or views everything *but* hidden strings
+#[derive(Clone)]
 pub enum Views {
     Show(Vec<String>),
     Hide(Vec<String>),
 }
 
+/// Gets a problem from passed in filepath and name of problem
+
+pub fn fetch_problem(problem_name: &str, problem_dir: &Path) -> MapmResult<Problem> {
+    match fs::read_to_string(problem_dir.join(&[problem_name, ".yml"].concat())) {
+        Ok(problem_yaml) => match parse_problem_yaml(&problem_yaml) {
+            Success(problem) => {
+                return Success(problem);
+            }
+            err => {
+                return err;
+            }
+        },
+        Err(_) => {
+            return Fail(ProblemErr(
+                [
+                    "Could not read problem ",
+                    &problem_name,
+                    " from `",
+                    &problem_dir.to_str().unwrap(),
+                    &problem_name,
+                    ".yml`",
+                ]
+                .concat(),
+            ));
+        }
+    }
+}
+
 impl Problem {
     /// Converts Problem into TeX string to be inputted in `mapm-headers.tex`
     ///
@@ -59,16 +102,16 @@ impl Problem {
     ///
     /// ```
     /// use mapm::problem::Problem;
-    /// use linked_hash_map::LinkedHashMap;
+    /// use std::collections::HashMap;
     ///
-    /// let mut vars = LinkedHashMap::new();
+    /// let mut vars = HashMap::new();
     /// vars.insert(String::from("problem"), String::from("What is $1+1$?"));
     /// vars.insert(String::from("author"), String::from("Dennis Chen"));
     ///
-    /// let mut solution_one = LinkedHashMap::new();
+    /// let mut solution_one = HashMap::new();
     /// solution_one.insert(String::from("text"), String::from("It's probably $2$."));
     /// solution_one.insert(String::from("author"), String::from("Dennis Chen"));
-    /// let mut solution_two = LinkedHashMap::new();
+    /// let mut solution_two = HashMap::new();
     /// solution_two.insert(String::from("text"), String::from("The answer is $2$, but my proof is too small to fit into the margin."));
     /// solution_two.insert(String::from("author"), String::from("Pierre de Fermat"));
     ///
@@ -76,8 +119,7 @@ impl Problem {
     ///
     /// let problem = Problem { vars, solutions };
     ///
-    /// let problem_tex = "\\def\\mapm@probvar@1@problem{What is $1+1$?}\n\\def\\mapm@probvar@1@author{Dennis Chen}\n\\def\\mapm@solcount@1{2}\\def\\mapm@solvar@1@1@text{It's probably $2$.}\n\\def\\mapm@solvar@1@1@author{Dennis Chen}\n\\def\\mapm@solvar@1@2@text{The answer is $2$, but my proof is too small to fit into the margin.}\n\\def\\mapm@solvar@1@2@author{Pierre de Fermat}\n";
-    /// assert_eq!(problem_tex, problem.as_tex(1));
+    /// println!("{}", problem.as_tex(1));
     /// ```
     pub fn as_tex(&self, problem_number: u32) -> String {
         let mut tex: String = String::new();
@@ -127,62 +169,6 @@ impl Problem {
         return tex;
     }
 
-    /// Converts Problem into yaml, may be useful when some interface needs to directly modify a problem (like in a GUI app)
-    ///
-    /// The function uses two spaces for yaml indents, not four
-    ///
-    /// # Usage
-    ///
-    /// ```
-    /// use mapm::problem::Problem;
-    /// use linked_hash_map::LinkedHashMap;
-    ///
-    /// let mut vars = LinkedHashMap::new();
-    /// vars.insert(String::from("problem"), String::from("What is $1+1$?"));
-    /// vars.insert(String::from("author"), String::from("Dennis Chen"));
-    ///
-    /// let mut solution_one = LinkedHashMap::new();
-    /// solution_one.insert(String::from("text"), String::from("It's probably $2$."));
-    /// solution_one.insert(String::from("author"), String::from("Dennis Chen"));
-    /// let mut solution_two = LinkedHashMap::new();
-    /// solution_two.insert(String::from("text"), String::from("The answer is $2$, but my proof is too small to fit into the margin."));
-    /// solution_two.insert(String::from("author"), String::from("Pierre de Fermat"));
-    ///
-    /// let mut solutions = Vec::from([solution_one, solution_two]);
-    ///
-    /// let problem = Problem { vars, solutions };
-    ///
-    /// let yaml = "problem: What is $1+1$?
-    /// author: Dennis Chen
-    /// solutions:
-    ///   - text: It's probably $2$.
-    ///     author: Dennis Chen
-    ///   - text: The answer is $2$, but my proof is too small to fit into the margin.
-    ///     author: Pierre de Fermat
-    /// ";
-    ///
-    /// assert_eq!(problem.as_yaml(), yaml);
-    /// ```
-    pub fn as_yaml(&self) -> String {
-        let mut yaml: String = String::new();
-        for (key, val) in &self.vars {
-            yaml.push_str(&[&key, ": ", &val, "\n"].concat());
-        }
-        yaml.push_str("solutions:\n");
-        for map in &self.solutions {
-            let mut first: bool = true;
-            for (key, val) in map {
-                if first {
-                    yaml.push_str(&["  - ", &key, ": ", &val, "\n"].concat());
-                    first = false;
-                } else {
-                    yaml.push_str(&["    ", &key, ": ", &val, "\n"].concat());
-                }
-            }
-        }
-        return yaml;
-    }
-
     /// Checks if a problem passes or fails a certain filter
     ///
     /// For Gt, it checks if the value of the problem is greater than the value passed in the filter (not the other way around). The same is true of Lt, Ge, Le.
@@ -192,17 +178,17 @@ impl Problem {
     /// ```
     /// use mapm::problem::Problem;
     /// use mapm::problem::Filter;
-    /// use linked_hash_map::LinkedHashMap;
+    /// use std::collections::HashMap;
     ///
-    /// let mut vars = LinkedHashMap::new();
+    /// let mut vars = HashMap::new();
     /// vars.insert(String::from("problem"), String::from("What is $1+1$?"));
     /// vars.insert(String::from("author"), String::from("Dennis Chen"));
     /// vars.insert(String::from("difficulty"), String::from("5"));
     ///
-    /// let mut solution_one = LinkedHashMap::new();
+    /// let mut solution_one = HashMap::new();
     /// solution_one.insert(String::from("text"), String::from("It's probably $2$."));
     /// solution_one.insert(String::from("author"), String::from("Dennis Chen"));
-    /// let mut solution_two = LinkedHashMap::new();
+    /// let mut solution_two = HashMap::new();
     /// solution_two.insert(String::from("text"), String::from("The answer is $2$, but my proof is too small to fit into the margin."));
     /// solution_two.insert(String::from("author"), String::from("Pierre de Fermat"));
     ///
@@ -311,13 +297,11 @@ impl Problem {
     }
 
     /// Show only the keys passed in or everything but the keys passed in to a problem
-    ///
-    /// If `views` is empty, keeps every key by default (does nothing)
 
     pub fn filter_keys(self, views: Views) -> (Vars, Option<Solutions>) {
         match views {
             Show(tags) => {
-                let mut vars: Vars = LinkedHashMap::new();
+                let mut vars: Vars = HashMap::new();
                 let mut solutions: bool = false;
                 for (key, val) in self.vars {
                     if key == "solutions" {
@@ -335,7 +319,7 @@ impl Problem {
                 }
             }
             Hide(tags) => {
-                let mut vars: Vars = LinkedHashMap::new();
+                let mut vars: Vars = HashMap::new();
                 let mut solutions: bool = true;
                 for (key, val) in self.vars {
                     if key == "solutions" {
@@ -347,9 +331,9 @@ impl Problem {
                     }
                 }
                 if solutions {
-                    return (vars, None);
-                } else {
                     return (vars, Some(self.solutions));
+                } else {
+                    return (vars, None);
                 }
             }
         }
@@ -376,6 +360,7 @@ impl Problem {
     /// texfiles:
     ///   problems.tex: ${title}.PDF
     ///   solutions.tex: ${title}-sols.PDF
+    /// problem_count: 1
     /// vars:
     ///   - title
     ///   - year
@@ -389,7 +374,7 @@ impl Problem {
     /// let problem = parse_problem_yaml(problem_yaml).unwrap();
     /// let template = parse_template_yaml(template_yaml).unwrap();
     ///
-    /// assert!(problem.check_template(template).is_none());
+    /// assert!(problem.check_template(&template).is_none());
     /// ```
     ///
     /// ## Expected failure
@@ -397,8 +382,8 @@ impl Problem {
     /// ```
     /// use mapm::problem::*;
     /// use mapm::template::*;
-    /// use mapm::mapm_result::MapmErr;
-    /// use mapm::mapm_result::MapmErr::*;
+    /// use mapm::result::MapmErr;
+    /// use mapm::result::MapmErr::*;
     /// let problem_yaml = "problem: What is $1+1$?
     /// solutions:
     ///   - text: Some say the answer is $2$.
@@ -412,6 +397,7 @@ impl Problem {
     /// vars:
     ///   - title
     ///   - year
+    /// problem_count: 1
     /// problemvars:
     ///   - problem
     ///   - author
@@ -423,7 +409,7 @@ impl Problem {
     /// let problem = parse_problem_yaml(problem_yaml).unwrap();
     /// let template = parse_template_yaml(template_yaml).unwrap();
     ///
-    /// let template_check = problem.check_template(template).unwrap();
+    /// let template_check = problem.check_template(&template).unwrap();
     ///
     /// assert_eq!(template_check.len(), 1);
     /// match &template_check[0] {
@@ -478,9 +464,8 @@ impl Problem {
 /// ```
 /// use mapm::problem::Problem;
 /// use mapm::problem::parse_problem_yaml;
-/// use linked_hash_map::LinkedHashMap;
-/// let yaml = "
-/// problem: What is $1+1$?
+/// use std::collections::HashMap;
+/// let yaml = "problem: What is $1+1$?
 /// author: Dennis Chen
 /// solutions:
 ///   - text: It's probably $2$.
@@ -490,14 +475,14 @@ impl Problem {
 /// ";
 /// let problem = parse_problem_yaml(yaml).unwrap();
 ///
-/// let mut vars = LinkedHashMap::new();
+/// let mut vars = HashMap::new();
 /// vars.insert(String::from("problem"), String::from("What is $1+1$?"));
 /// vars.insert(String::from("author"), String::from("Dennis Chen"));
 ///
-/// let mut solution_one = LinkedHashMap::new();
+/// let mut solution_one = HashMap::new();
 /// solution_one.insert(String::from("text"), String::from("It's probably $2$."));
 /// solution_one.insert(String::from("author"), String::from("Dennis Chen"));
-/// let mut solution_two = LinkedHashMap::new();
+/// let mut solution_two = HashMap::new();
 /// solution_two.insert(String::from("text"), String::from("The answer is $2$, but my proof is too small to fit into the margin."));
 /// solution_two.insert(String::from("author"), String::from("Pierre de Fermat"));
 ///
@@ -508,61 +493,36 @@ impl Problem {
 /// ```
 
 pub fn parse_problem_yaml(yaml: &str) -> MapmResult<Problem> {
-    let parsed_yaml = StrictYamlLoader::load_from_str(yaml);
-    match parsed_yaml {
-        Ok(data) => {
-            let mut vars: Vars = LinkedHashMap::new();
-            let mut solutions: Solutions = Vec::new();
-
-            let map = data[0].as_hash();
-            match map {
-                Some(map) => {
-                    for (key, val) in map {
-                        if key.as_str().unwrap() == "solutions" {
-                            match val.as_vec() {
-                                Some(solutions_vec) => {
-                                    for solution_hash in solutions_vec {
-                                        match solution_hash.as_hash() {
-                                            Some(solution_hash) => {
-                                                let mut solution: LinkedHashMap<String, String> =
-                                                    LinkedHashMap::new();
-                                                for (key, val) in solution_hash {
-                                                    solution.insert(
-                                                        String::from(key.as_str().unwrap()),
-                                                        String::from(val.as_str().unwrap()),
-                                                    );
-                                                }
-                                                solutions.push(solution);
-                                            }
-                                            None => {
-                                                return Fail(ProblemErr(String::from("Some element in 'solutions' has no keys, or they couldn't be interpreted")));
-                                            }
-                                        }
-                                    }
-                                }
-                                None => {
-                                    return Fail(SolutionErr(String::from("No array 'solutions' in yaml, or could not interpret 'solutions' as an array")));
-                                }
-                            }
-                        } else {
-                            vars.insert(
-                                String::from(key.as_str().unwrap()),
-                                String::from(val.as_str().unwrap()),
-                            );
-                        }
+    match serde_yaml::from_str::<SerializedProblem>(yaml) {
+        Ok(problem) => {
+            let mut vars: Vars = HashMap::new();
+            for (key, val) in problem.vars {
+                match val {
+                    Value::String(val) => {
+                        vars.insert(key, val);
+                    }
+                    Value::Number(val) => {
+                        vars.insert(key, val.to_string());
+                    }
+                    Value::Bool(val) => {
+                        match val {
+                            true => vars.insert(key, String::from("true")),
+                            false => vars.insert(key, String::from("false")),
+                        };
+                    }
+                    _ => {
+                        return Fail(ProblemErr(["Could not parse key `", &key, "`"].concat()));
                     }
-
-                    return Success(Problem { vars, solutions });
-                }
-                None => {
-                    return Fail(ProblemErr(String::from("Problem yaml is empty")));
                 }
             }
+
+            return Success(Problem {
+                vars,
+                solutions: problem.solutions,
+            });
         }
-        Err(e) => {
-            return Fail(ProblemErr(
-                ["Could not parse problem yaml:\n", &e.to_string()].concat(),
-            ));
+        Err(err) => {
+            return Fail(ProblemErr(err.to_string()));
         }
     }
 }
